@@ -4,11 +4,10 @@
 # Author: Ilya Mashchenko (ilyam8)
 # User Memory Stat Author: Guido Scatena (scatenag)
 
-import subprocess
-import threading
 import os
 import pwd
-
+import subprocess
+import threading
 import xml.etree.ElementTree as et
 
 from bases.FrameworkServices.SimpleService import SimpleService
@@ -23,6 +22,7 @@ EMPTY_ROW_LIMIT = 500
 POLLER_BREAK_ROW = '</nvidia_smi_log>'
 
 PCI_BANDWIDTH = 'pci_bandwidth'
+PCI_BANDWIDTH_PERCENT = 'pci_bandwidth_percent'
 FAN_SPEED = 'fan_speed'
 GPU_UTIL = 'gpu_utilization'
 MEM_UTIL = 'mem_utilization'
@@ -32,12 +32,14 @@ BAR_USAGE = 'bar1_mem_usage'
 TEMPERATURE = 'temperature'
 CLOCKS = 'clocks'
 POWER = 'power'
+POWER_STATE = 'power_state'
 PROCESSES_MEM = 'processes_mem'
 USER_MEM = 'user_mem'
 USER_NUM = 'user_num'
 
 ORDER = [
     PCI_BANDWIDTH,
+    PCI_BANDWIDTH_PERCENT,
     FAN_SPEED,
     GPU_UTIL,
     MEM_UTIL,
@@ -47,12 +49,31 @@ ORDER = [
     TEMPERATURE,
     CLOCKS,
     POWER,
+    POWER_STATE,
     PROCESSES_MEM,
     USER_MEM,
     USER_NUM,
 ]
 
+# https://docs.nvidia.com/gameworks/content/gameworkslibrary/coresdk/nvapi/group__gpupstate.html
+POWER_STATES = ['P' + str(i) for i in range(0, 16)]
 
+# PCI Transfer data rate in gigabits per second (Gb/s) per generation
+PCI_SPEED = {
+  "1": 2.5,
+  "2": 5,
+  "3": 8,
+  "4": 16,
+  "5": 32
+}
+# PCI encoding per generation
+PCI_ENCODING = {
+  "1": 2/10,
+  "2": 2/10,
+  "3": 2/130,
+  "4": 2/130,
+  "5": 2/130
+}
 def gpu_charts(gpu):
     fam = gpu.full_name()
 
@@ -64,6 +85,13 @@ def gpu_charts(gpu):
                 ['tx_util', 'tx', 'absolute', 1, -1],
             ]
         },
+        PCI_BANDWIDTH_PERCENT: {
+            'options': [None, 'PCI Express Bandwidth Percent', 'percentage', fam, 'nvidia_smi.pci_bandwidth_percent', 'area'],
+            'lines': [
+                ['rx_util_percent', 'rx_percent'],
+                ['tx_util_percent', 'tx_percent'],
+            ]
+        },
         FAN_SPEED: {
             'options': [None, 'Fan Speed', 'percentage', fam, 'nvidia_smi.fan_speed', 'line'],
             'lines': [
@@ -125,6 +153,10 @@ def gpu_charts(gpu):
                 ['power_draw', 'power', 'absolute', 1, 100],
             ]
         },
+        POWER_STATE: {
+            'options': [None, 'Power State', 'state', fam, 'nvidia_smi.power_state', 'line'],
+            'lines': [['power_state_' + v.lower(), v, 'absolute'] for v in POWER_STATES]
+        },
         PROCESSES_MEM: {
             'options': [None, 'Memory Used by Each Process', 'MiB', fam, 'nvidia_smi.processes_mem', 'stacked'],
             'lines': []
@@ -318,6 +350,24 @@ class GPU:
     def full_name(self):
         return 'gpu{0} {1}'.format(self.num, self.name())
 
+    @handle_attr_error
+    def pci_link_gen(self):
+        return self.root.find('pci').find('pci_gpu_link_info').find('pcie_gen').find('max_link_gen').text
+
+    @handle_attr_error
+    def pci_link_width(self):
+        return self.root.find('pci').find('pci_gpu_link_info').find('link_widths').find('max_link_width').text.split('x')[0]
+
+    def pci_bw_max(self):
+        link_gen = self.pci_link_gen()
+        link_width = int(self.pci_link_width())
+        if link_gen not in PCI_SPEED or link_gen not in PCI_ENCODING or not link_width:
+            return None
+        # Maximum PCIe Bandwidth = SPEED * WIDTH * (1 - ENCODING) - 1Gb/s.
+        # see details https://enterprise-support.nvidia.com/s/article/understanding-pcie-configuration-for-maximum-performance
+        # return max bandwidth in kilobytes per second (kB/s)
+        return (PCI_SPEED[link_gen] * link_width * (1- PCI_ENCODING[link_gen]) - 1) * 1000 * 1000 / 8
+
     @handle_attr_error
     def rx_util(self):
         return self.root.find('pci').find('rx_util').text.split()[0]
@@ -382,6 +432,10 @@ class GPU:
     def mem_clock(self):
         return self.root.find('clocks').find('mem_clock').text.split()[0]
 
+    @handle_attr_error
+    def power_state(self):
+        return str(self.root.find('power_readings').find('power_state').text.split()[0])
+
     @handle_value_error
     @handle_attr_error
     def power_draw(self):
@@ -426,6 +480,22 @@ class GPU:
             'mem_clock': self.mem_clock(),
             'power_draw': self.power_draw(),
         }
+
+        pci_bw_max = self.pci_bw_max()
+        if not pci_bw_max:
+            data['rx_util_percent'] = 0
+            data['tx_util_percent'] = 0
+        else :
+            data['rx_util_percent'] = str(int(int(self.rx_util())*100/self.pci_bw_max()))
+            data['tx_util_percent'] = str(int(int(self.tx_util())*100/self.pci_bw_max()))
+
+
+        for v in POWER_STATES:
+            data['power_state_' + v.lower()] = 0
+        p_state = self.power_state()
+        if p_state:
+            data['power_state_' + p_state.lower()] = 1
+
         processes = self.processes() or []
         users = set()
         for p in processes:
@@ -450,7 +520,7 @@ class Service(SimpleService):
         self.order = list()
         self.definitions = dict()
         self.loop_mode = configuration.get('loop_mode', True)
-        poll = int(configuration.get('poll_seconds', 1))
+        poll = int(configuration.get('poll_seconds', self.get_update_every()))
         self.exclude_zero_memory_users = configuration.get('exclude_zero_memory_users', False)
         self.poller = NvidiaSMIPoller(poll)
 
